/*
  Rodent, a UCI chess playing engine derived from Sungorus 1.4
  Copyright (C) 2009-2011 Pablo Vazquez (Sungorus author)
  Copyright (C) 2011-2012 Pawel Koziol

  Rodent is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published 
  by the Free Software Foundation, either version 3 of the License, 
  or (at your option) any later version.

  Rodent is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty 
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "bitboard.h"
#include "bitmasks.h"

// NOTE: occluded attack functions set also start square to 1
U64 RookAttacks(U64 bbStart, U64 bbBlock) {
    U64 result = FillOcclNorth(bbStart, bbBlock)
		       | FillOcclSouth(bbStart, bbBlock)
			   | FillOcclEast(bbStart, bbBlock)
			   | FillOcclWest(bbStart, bbBlock);
	return result;
}

U64 BishAttacks(U64 bbStart, U64 bbBlock) {
    U64 result = FillOcclNE(bbStart, bbBlock)
		       | FillOcclNW(bbStart, bbBlock)
			   | FillOcclSE(bbStart, bbBlock)
			   | FillOcclSW(bbStart, bbBlock);
	return result;
}

U64 QueenAttacks(U64 bbStart, U64 bbBlock) {
    return BishAttacks(bbStart, bbBlock) | RookAttacks(bbStart,bbBlock);
}

U64 FillOcclSouth(U64 bbStart, U64 bbBlock) {
   bbStart |= bbBlock & (bbStart >>  8);
   bbBlock &=           (bbBlock >>  8);
   bbStart |= bbBlock & (bbStart >> 16);
   bbBlock &=           (bbBlock >> 16);
   bbStart |= bbBlock & (bbStart >> 32);
   return bbStart;
}
 
U64 FillOcclNorth(U64 bbStart, U64 bbBlock) {
   bbStart |= bbBlock & (bbStart <<  8);
   bbBlock &=           (bbBlock <<  8);
   bbStart |= bbBlock & (bbStart << 16);
   bbBlock &=           (bbBlock << 16);
   bbStart |= bbBlock & (bbStart << 32);
   return bbStart;
}
 
U64 FillOcclEast(U64 bbStart, U64 bbBlock) {

   bbBlock &= bbNotA;
   bbStart |= bbBlock & (bbStart << 1);
   bbBlock &=           (bbBlock << 1);
   bbStart |= bbBlock & (bbStart << 2);
   bbBlock &=           (bbBlock << 2);
   bbStart |= bbBlock & (bbStart << 4);
   return bbStart;
}
 
U64 FillOcclNE(U64 bbStart, U64 bbBlock) {
   bbBlock &= bbNotA;
   bbStart |= bbBlock & (bbStart <<  9);
   bbBlock &=           (bbBlock <<  9);
   bbStart |= bbBlock & (bbStart << 18);
   bbBlock &=           (bbBlock << 18);
   bbStart |= bbBlock & (bbStart << 36);
   return bbStart;
}
 
U64 FillOcclSE(U64 bbStart, U64 bbBlock) {
   bbBlock &= bbNotA;
   bbStart |= bbBlock & (bbStart >>  7);
   bbBlock &=           (bbBlock >>  7);
   bbStart |= bbBlock & (bbStart >> 14);
   bbBlock &=           (bbBlock >> 14);
   bbStart |= bbBlock & (bbStart >> 28);
   return bbStart;
}
 
U64 FillOcclWest(U64 bbStart, U64 bbBlock) {
   bbBlock &= bbNotH;
   bbStart |= bbBlock & (bbStart >> 1);
   bbBlock &=           (bbBlock >> 1);
   bbStart |= bbBlock & (bbStart >> 2);
   bbBlock &=           (bbBlock >> 2);
   bbStart |= bbBlock & (bbStart >> 4);
   return bbStart;
}
 
U64 FillOcclSW(U64 bbStart, U64 bbBlock) {
   bbBlock &= bbNotH;
   bbStart |= bbBlock & (bbStart >>  9);
   bbBlock &=           (bbBlock >>  9);
   bbStart |= bbBlock & (bbStart >> 18);
   bbBlock &=           (bbBlock >> 18);
   bbStart |= bbBlock & (bbStart >> 36);
   return bbStart;
}
 
U64 FillOcclNW(U64 bbStart, U64 bbBlock) {
   bbBlock &= bbNotH;
   bbStart |= bbBlock & (bbStart <<  7);
   bbBlock &=           (bbBlock <<  7);
   bbStart |= bbBlock & (bbStart << 14);
   bbBlock &=           (bbBlock << 14);
   bbStart |= bbBlock & (bbStart << 28);
   return bbStart;
}