/*
  Rodent, a UCI chess playing engine derived from Sungorus 1.4
  Copyright (C) 2009-2011 Pablo Vazquez (Sungorus author)
  Copyright (C) 2011-2012 Pawel Koziol

  Rodent is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published
  by the Free Software Foundation, either version 3 of the License,
  or (at your option) any later version.

  Rodent is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* Compiler switches activating faster instructions for newer 64-bit architectures
   - patch supplied by Dann Corbit, 17.02.2012
*/

// #define USE_FIRST_ONE_INTRINSICS
// #define USE_MM_POPCNT

#include <stdint.h>

typedef  uint8_t	U8,  *PU8;
typedef  uint16_t	U16, *PU16;
typedef  uint32_t	U32, *PU32;
typedef uint64_t U64, *PU64;



typedef signed char	S8,  *PS8;
typedef signed short	S16, *PS16;
typedef signed long	S32, *PS32;


//typedef unsigned long long U64;

#define SqBb(x)         ((U64)1 << (x)) // returns bitboard with a bit of square "x" set

// Kindergarten bitboard move generator by Pablo Vazquez;
// data used by it are generated by InitKindergartenBitboards in bb_init.c

#define RankIndex(o, x) (((o) >> ((070 & (x)) + 1)) & 63)
#define FileIndex(o, x) (((bbFILE_A & ((o) >> File(x))) * (U64)0x0204081020408000) >> 58)
#define DiagIndex(o, x) ((((o) & bbLineMask[2][x]) * bbFILE_B) >> 58)
#define AntiIndex(o, x) ((((o) & bbLineMask[3][x]) * bbFILE_B) >> 58)

#define HorAttacks(o, x) attacks[HOR]    [x][RankIndex(o, x)]
#define VerAttacks(o, x) attacks[VER]    [x][FileIndex(o, x)]
#define AHDAttacks(o, x) attacks[DIAG_AH][x][DiagIndex(o, x)]
#define HADAttacks(o, x) attacks[DIAG_HA][x][AntiIndex(o, x)]

#define RAttacks(o, x)  (HorAttacks(o, x) | VerAttacks(o, x))
#define BAttacks(o, x)  (AHDAttacks(o, x) | HADAttacks(o, x))
#define QAttacks(o, x)  (RAttacks(o, x)   | BAttacks(o, x))

#ifdef USE_FIRST_ONE_INTRINSICS
#include <intrin.h>
#pragma intrinsic(_BitScanForward64)

static int __forceinline FirstOne(U64 x) {
    unsigned long index = -1;
    _BitScanForward64(&index, x);
    return index;
}
#else
#define FirstOne(x)     bitTable[(((x) & (~(x) + 1)) * (U64)0x0218A392CD3D5DBF) >> 58] // first "1" in a bitboard
#endif

void PrintBb(U64 bbTest);
extern int PopCnt(U64);
extern int PopCnt15(U64 bb);
extern int PopCntSparse(U64);

extern U64 ShiftNorth(U64 bb);
extern U64 ShiftSouth(U64 bb);
extern U64 ShiftWest(U64 bb);
extern U64 ShiftEast(U64 bb);
extern U64 ShiftNE(U64 bb);
extern U64 ShiftNW(U64 bb);
extern U64 ShiftSE(U64 bb);
extern U64 ShiftSW(U64 bb);

U64 FillKing(U64 bb);
U64 FillNorth(U64 bb);
U64 FillSouth(U64 bb);
U64 FillWest(U64 bb);
U64 FillEast(U64 bb);
U64 FillNW(U64 bb);
U64 FillNE(U64 bb);
U64 FillSW(U64 bb);
U64 FillSE(U64 bb);

U64 FillNorthExcl(U64 bb);
U64 FillSouthExcl(U64 bb);
U64 FillWestExcl(U64 bb);
U64 FillEastExcl(U64 bb);
U64 FillNWExcl(U64 bb);
U64 FillNEExcl(U64 bb);
U64 FillSWExcl(U64 bb);
U64 FillSEExcl(U64 bb);

U64 FillOcclNorth(U64 bbStart, U64 bbBlock);
U64 FillOcclSouth(U64 bbStart, U64 bbBlock);
U64 FillOcclEast(U64 bbStart, U64 bbBlock);
U64 FillOcclWest(U64 bbStart, U64 bbBlock);
U64 FillOcclNE(U64 bbStart, U64 bbBlock);
U64 FillOcclNW(U64 bbStart, U64 bbBlock);
U64 FillOcclSE(U64 bbStart, U64 bbBlock);
U64 FillOcclSW(U64 bbStart, U64 bbBlock);

U64 RookAttacks(U64 bbStart, U64 bbBlock);
U64 BishAttacks(U64 bbStart, U64 bbBlock);
U64 QueenAttacks(U64 bbStart, U64 bbBlock);

U64 GetFrontSpan(U64 bb, int side);
U64 GetRearSpan(U64 bb, int side);
U64 GetWPControl(U64 bb);
U64 GetBPControl(U64 bb);
U64 GetPawnAttacks(int side, U64 bb);

void InitKindergartenBitboards(void);
void InitPawnAttacks(void);
void InitKnightAttacks(void);
void InitKingAttacks(void);
