/*
  Rodent, a UCI chess playing engine derived from Sungorus 1.4
  Copyright (C) 2009-2011 Pablo Vazquez (Sungorus author)
  Copyright (C) 2011-2012 Pawel Koziol

  Rodent is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published 
  by the Free Software Foundation, either version 3 of the License, 
  or (at your option) any later version.

  Rodent is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty 
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

struct sPawnHashEntry {
  U64 pawnKey;
  int mgPawns;
  int egPawns;
};

#define PAWN_HASH_SIZE  512 * 512

struct sEvaluator {
private:
  int attScore[2];      // king attack scores
  int mgMobility[2];    // midgame mobility scores
  int egMobility[2];    // endgame mobility scores
  int mgMisc[2];        // miscelanneous midgame scores
  int egMisc[2];        // miscelanneous endgame scores
  //int mgPassers[2];
  //int egPassers[2];
  int pawnScoreMg[2];   // midgame pawn structure scores
  int pawnScoreEg[2];   // endgame pawn structure scores
  U64 bbKingZone[2];    // bitboard of squares near enemy king
  U64 bbShieldZone[2];  // bitboard of squares used in rudimentary pawn shield eval
  U64 bbPawnControl[2];
  U64 bbPawnCanControl[2];
  U64 kingDiagChecks[2];
  U64 kingStraightChecks[2];
  U64 kingKnightChecks[2];
  int woodCount[2];
  int attCount[2];
  int mgFact,  egFact;  // scaling factors
  int mgScore, egScore; // partial midgame and endgame scores (to be scaled)
  int degradation;      // for scaling down the score in endgames that are hard to win

  sPawnHashEntry PawnTT[PAWN_HASH_SIZE]; // pawn transposition table

  int GetMaterialScore(sPosition *p);
  void AddMobility(int pc, int side, int cnt);
  void AddMiscOne(int side, int val);
  void AddMiscTwo(int side, int mg, int eg);
  int AdjustMaterial(sPosition *p);
  void InitStatic(void);            
  void InitDynamic(void);            
  void InitPawnData(sPosition *p); // initialization of pawn control / potential pawn control
  void SetKingZones(sPosition *p);
  void SetScaleFactor(sPosition *p);
  void SetScoreDegradation(sPosition *p, int score);
  int Interpolate(void);
  void PawnScore(sPosition *, int);
  void EvalPawns(sPosition *p);
  void EvalPassers(sPosition *p, int side);
  void ScoreN(sPosition *p, int *attCount, int *woodCount, int side);
  void ScoreB(sPosition *p, int *attCount, int *woodCount, int side);
  void ScoreR(sPosition *p, int *attCount, int *woodCount, int side);
  void ScoreQ(sPosition *p, int *attCount, int *woodCount, int side);
  void EvalKings(sPosition *p);
  int  EvalTrappedKnight(sPosition *p);
  int  EvalTrappedBishop(sPosition *p);
  int  EvalTrappedRook(sPosition *p);
public:
  int Normalize(int val, int limit);
  void ScaleValue(int * value, int factor);
  int ReturnMaterial(sPosition *);
  int ReturnFast(sPosition *);
  int Return(sPosition *, int alpha, int beta);
};

extern struct sEvaluator Eval;

int BishopsAreDifferent(sPosition *p);
int NotOnBishColor(sPosition * p, int bishSide, int sq);
int MaterialMinor(sPosition *p, int side);
int MaterialKnight(sPosition *p, int side);
int MaterialBishop(sPosition *p, int side);
int MaterialRook(sPosition *p, int side);
int MaterialRookMinor(sPosition *p, int side);
int MaterialQueen(sPosition *p, int side);
int MaterialQueenMinor(sPosition *p, int side);
int MaterialBB(sPosition *p, int side);
int MaterialNN(sPosition *p, int side);
int MaterialBN(sPosition *p, int side);