/*
  Rodent, a UCI chess playing engine derived from Sungorus 1.4
  Copyright (C) 2009-2011 Pablo Vazquez (Sungorus author)
  Copyright (C) 2011-2012 Pawel Koziol

  Rodent is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published 
  by the Free Software Foundation, either version 3 of the License, 
  or (at your option) any later version.

  Rodent is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty 
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "bitboard.h"
#include "data.h"
#include "rodent.h"
#include "eval.h"

void sEvaluator::EvalPawns(sPosition *p)
{
  int pawnHash = p->pawnKey % PAWN_HASH_SIZE;

  if ( PawnTT[pawnHash].pawnKey == p->pawnKey ) {
	  mgScore += PawnTT[pawnHash].mgPawns;
	  egScore += PawnTT[pawnHash].egPawns;
  }
  else
  {
  PawnScore(p, WHITE); // pawn eval
  PawnScore(p, BLACK);  
  mgScore += (pawnScoreMg[WHITE] - pawnScoreMg[BLACK]);
  egScore += (pawnScoreEg[WHITE] - pawnScoreEg[BLACK]);

  PawnTT[pawnHash].pawnKey = p->pawnKey;
  PawnTT[pawnHash].mgPawns = (pawnScoreMg[WHITE] - pawnScoreMg[BLACK]);
  PawnTT[pawnHash].egPawns = (pawnScoreEg[WHITE] - pawnScoreEg[BLACK]);
  }
}

void sEvaluator::PawnScore(sPosition *p, int side)
{
  int from; 
  int flagIsOpen;
  U64 bbPieces = bbPc(p, side, P);
  U64 bbFrontSpan;
 
  while (bbPieces) {
    from = FirstOne(bbPieces);
	bbFrontSpan = GetFrontSpan(SqBb(from), side );
	flagIsOpen = ( ( bbFrontSpan & bbPc(p, Opp(side), P) ) == 0 );

	// doubled pawn
	if ( bbFrontSpan & bbPc(p, side, P) ) {
	  pawnScoreMg[side] += Data.doubledPawnMg;
	  pawnScoreEg[side] += Data.doubledPawnEg;
	}
  
	// passed pawn
	/**/
	if (!(bbPassedMask[side][from] & bbPc(p, Opp(side), P))) {
		if ( !(bbFrontSpan & bbPc(p, side, P)) ) {
	  pawnScoreMg[side] += Data.passersMg[side][from];	  
      pawnScoreEg[side] += Data.passersEg[side][from];	  
		}
	}
	/**/

	// weak pawn
	if (!(bbPawnSupport[side][from] & bbPc(p, side, P))) 
	{
		if (!(bbAdjacentMask[File(from)] & bbPc(p, side, P))) // isolated
		{ 
		   pawnScoreMg[side] += Data.isolatedMg[side][from];
		   if (flagIsOpen) 
			   pawnScoreMg[side] += Data.pawnIsolatedOnOpen;
		   pawnScoreEg[side] += Data.isolatedEg[side][from];
		}
		else // backward
		{
		   pawnScoreMg[side] += Data.backwardMg[side][from];
		   if (flagIsOpen) 
			   pawnScoreMg[side] += Data.pawnBackwardOnOpen;
		   pawnScoreEg[side] += Data.backwardEg[side][from];
		}
	}

	bbPieces &= bbPieces - 1;
  }
}

/*
void sEvaluator::EvalPassers(sPosition *p, int side)
{
  int from, next; 
  U64 bbPieces = bbPc(p, side, P);
 
  while (bbPieces) {
    from = FirstOne(bbPieces);
  
	// passed pawn
	if (!(bbPassedMask[side][from] & bbPc(p, Opp(side), P))) {
	  mgPassers[side] += Data.passersMg[side][from];	  
      egPassers[side] += Data.passersEg[side][from];

	  egPassers[side] += Data.distance[from][p->kingSquare[side]];

	  // passer is blocked
	  if (side == WHITE) next = from + 8;
	  else               next = from - 8;

	  if ( SqBb(next) & p->bbCl[Opp(side)] ) {
	    mgPassers[side] -= ( Data.passersMg[side][from] / 3);	  
        egPassers[side] -= ( Data.passersEg[side][from] / 3);
	  }
	  
	  else {
		  if ( (SqBb(next) & p->bbCl[Opp(side)] ) == 0 ) {
			  if (Swap(p, from, next) > -1 ) {
	    mgPassers[side] += ( Data.passersMg[side][from] / 5);	  
        egPassers[side] += ( Data.passersEg[side][from] / 5);

			  }
		  }
	  }
	  

	  // TODO: connected passers	  
	}


	bbPieces &= bbPieces - 1;
  }
}
*/