/*
  Rodent, a UCI chess playing engine derived from Sungorus 1.4
  Copyright (C) 2009-2011 Pablo Vazquez (Sungorus author)
  Copyright (C) 2011-2012 Pawel Koziol

  Rodent is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published 
  by the Free Software Foundation, either version 3 of the License, 
  or (at your option) any later version.

  Rodent is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty 
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* GenCache class saves mobility bitboards of sliding pieces for a given 
   square, allowing to reuse them whenever occupancy along relevant rays 
   has not changed. It is used both in eval and move generation and gives 
   a minor speedup. */

#include "rodent.h"
#include "bitboard.h"
#include "bitmasks.h"
#include "gencache.h"

#define USE_GEN_CACHE

U64 sGenCache::GetRookMob(U64 bbOccupied, int sq) 
{
#ifdef USE_GEN_CACHE
	// generate mobility bitboard or read it from a table
	U64 bbRelevantOcc = bbOccupied & bbRAttacksOnEmpty[sq];
	if (bbRelevantOcc == bbROcc[sq])
		return bbRMob[sq];
	else {
	  U64 bbControl  = RAttacks(bbOccupied, sq);  
	  bbRMob[sq] = bbControl;
	  bbROcc[sq] = bbRelevantOcc;
	  return bbControl;
	}	
#else
	return RAttacks(bbOccupied, sq);  
#endif
}

U64 sGenCache::GetBishMob(U64 bbOccupied, int sq) {
#ifdef USE_GEN_CACHE
	U64 bbRelevantOcc = bbOccupied & bbBAttacksOnEmpty[sq];
	if (bbRelevantOcc == bbBOcc[sq])
		return bbBMob[sq];
	else {
	  U64 bbControl  = BAttacks(bbOccupied, sq);  
	  bbBMob[sq] = bbControl;
	  bbBOcc[sq] = bbRelevantOcc;
	  return bbControl;
	}	
#else
	return BAttacks(bbOccupied, sq);  
#endif
}

U64 sGenCache::GetQueenMob(U64 bbOccupied, int sq) {
    return ( GetBishMob(bbOccupied, sq) | GetRookMob(bbOccupied, sq) );
}